﻿using April.Data;
using April.Models;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using NuGet.Packaging.Signing;
using Syncfusion.EJ2.Base;
using Syncfusion.EJ2.Linq;
using System.Diagnostics;
using System.Linq;

namespace April.Controllers
{
    public class UsersController : Controller
    {
        private readonly ILogger<HomeController> _logger;
        private readonly CosmosDbContext _cosmosDbContext;

        public UsersController(ILogger<HomeController> logger, CosmosDbContext cosmosDbContext)
        {
            _logger = logger;
            _cosmosDbContext = cosmosDbContext;
        }

        public IActionResult Deposits()
        {
            return View();
        }

        public IActionResult DepositsUrlAdaptor([FromBody] DataManagerRequest dm)
        {
            return UrlAdaptor(_cosmosDbContext.Deposits, dm, new string[] { "Value" });
        }

        public IActionResult Trades()
        {
            return View();
        }

        public IActionResult TradesUrlAdaptor([FromBody] DataManagerRequest dm)
        {
            return UrlAdaptor(_cosmosDbContext.Trades, dm, new string[] { "Group", "Order" });
        }

        public IActionResult TradesMT5()
        {
            return View();
        }

        public IActionResult TradesMT5UrlAdaptor([FromBody] DataManagerRequest dm)
        {
            return UrlAdaptor(_cosmosDbContext.TradingAccounts, dm, new string[] { "Group" });
        }

        public IActionResult TradingAccounts()
        {
            return View();
        }

        public IActionResult TradingAccountsUrlAdaptor([FromBody] DataManagerRequest dm)
        {
            return UrlAdaptor(_cosmosDbContext.TradingAccounts, dm, new string[] { "Group" });
        }

        public IActionResult Withdrawals()
        {
            return View();
        }

        public IActionResult WithdrawalsUrlAdaptor([FromBody] DataManagerRequest dm)
        {
            return UrlAdaptor(_cosmosDbContext.Withdrawals, dm, new string[] { "Value" });
        }

        [ResponseCache(Duration = 0, Location = ResponseCacheLocation.None, NoStore = true)]
        public IActionResult Error()
        {
            return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
        }

        private IActionResult UrlAdaptor<TEntity>(DbSet<TEntity> source, DataManagerRequest dm, string[] reservedWords) where TEntity : class
        {
            IQueryable<TEntity> dataSource = source;
            QueryableOperation operation = new();

            if (dm.Search != null && 0 < dm.Search.Count)
            {
                List<string> whereAndConditions = new();

                foreach (var searchKey in dm.Search[0].Key.Split(' '))
                {
                    List<string> whereOrConditions = new();

                    foreach (var field in dm.Search[0].Fields)
                    {
                        if (typeof(TEntity).GetProperty(field).PropertyType == typeof(string))
                            whereOrConditions.Add($"CONTAINS(LOWER(c.{field}), '{searchKey.ToLower()}')");

                        else if (typeof(TEntity).GetProperty(field).PropertyType == typeof(long))
                            whereOrConditions.Add($"CONTAINS(LOWER(ToString(c.{field})), '{searchKey.ToLower()}')");

                        else if (typeof(TEntity).GetProperty(field).PropertyType == typeof(double))
                            whereOrConditions.Add($"CONTAINS(LOWER(ToString(c.{field})), '{searchKey.ToLower()}')");

                        else if (typeof(TEntity).GetProperty(field).PropertyType == typeof(DateTimeOffset))
                            whereOrConditions.Add($"CONTAINS(LEFT(REPLACE(DateTimeAdd('hour', 9, LEFT(REPLACE(c.{field}, 'T', ' '), 19)), 'T', ' '), 19), '{searchKey.ToLower()}')");
                    }

                    foreach (var reservedWord in reservedWords)
                        for (int i = 0; i < whereOrConditions.Count; i++)
                            whereOrConditions[i] = whereOrConditions[i].Replace($"c.{reservedWord}", $"c['{reservedWord}']");

                    whereAndConditions.Add($"({string.Join(" \r\nOR ", whereOrConditions)})");
                }

                dataSource = CosmosQueryableExtensions.FromSqlRaw(
                    source
                    , $"SELECT * FROM c WHERE \r\n{string.Join(" \r\nAND ", whereAndConditions)}"
                );
            }

            if (dm.Where != null && 0 < dm.Where.Count)
                try { dataSource = operation.PerformFiltering(dataSource, dm.Where, dm.Where[0].Operator); }
                catch (Exception ex)
                {
                }

            int count = dataSource.Count();

            if (dm.Sorted != null && 0 < dm.Sorted.Count)
                dataSource = operation.PerformSorting(dataSource, dm.Sorted);

            if (dm.Skip != 0)
                dataSource = operation.PerformSkip(dataSource, dm.Skip);

            if (dm.Take != 0)
                dataSource = operation.PerformTake(dataSource, dm.Take);

            return dm.RequiresCounts ? Json(new { result = dataSource, count = count }) : Json(dataSource);
        }
    }
}