﻿using April.Models;
using Microsoft.EntityFrameworkCore;

namespace April.Data
{
    public class CosmosDbContext : DbContext
    {
        public CosmosDbContext(DbContextOptions<CosmosDbContext> options)
            : base(options)
        {
        }

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            base.OnModelCreating(modelBuilder);

            modelBuilder.HasDefaultContainer("TradingAccounts");

            modelBuilder.Entity<DepositData>().Property(x => x.Id).ToJsonProperty("id");
            modelBuilder.Entity<DepositData>().Property(x => x.PartitionKey).ToJsonProperty("partitionKey");
            modelBuilder.Entity<DepositData>().ToContainer("Deposits").HasNoDiscriminator();

            modelBuilder.Entity<TradeData>().Property(x => x.Id).ToJsonProperty("id");
            modelBuilder.Entity<TradeData>().Property(x => x.PartitionKey).ToJsonProperty("partitionKey");
            modelBuilder.Entity<TradeData>().ToContainer("Trades").HasNoDiscriminator();

            modelBuilder.Entity<TradingAccountData>().Property(x => x.Id).ToJsonProperty("id");
            modelBuilder.Entity<TradingAccountData>().Property(x => x.PartitionKey).ToJsonProperty("partitionKey");
            modelBuilder.Entity<TradingAccountData>().ToContainer("TradingAccounts").HasNoDiscriminator();

            modelBuilder.Entity<WithdrawalData>().Property(x => x.Id).ToJsonProperty("id");
            modelBuilder.Entity<WithdrawalData>().Property(x => x.PartitionKey).ToJsonProperty("partitionKey");
            modelBuilder.Entity<WithdrawalData>().ToContainer("Withdrawals").HasNoDiscriminator();
        }

        public DbSet<DepositData> Deposits { get; set; }
        public DbSet<TradeData> Trades { get; set; }
        public DbSet<TradingAccountData> TradingAccounts { get; set; }
        public DbSet<WithdrawalData> Withdrawals { get; set; }
    }
}